/* This file is part of KsirK.
   Copyright (C) 2001-2007 Gael de Chalendar <kleag@free.fr>

   KsirK is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation, version 2.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA
*/

/**
  * @file This header defines data that are defined here to ease their change
  * begin                : dim sep 15 2002 
  */

#ifndef KSIRK_GLOBAL_H
#define KSIRK_GLOBAL_H

#define ONU_FILE_FORMAT_VERSION "3.0"
#define SAVE_GAME_FILE_FORMAT_VERSION "1.6"

#define ID_NO_STATUS_MSG 0
#define ID_STATUS_MSG 1
#define ID_STATUS_MSG2 2

#define CM_CANCEL    "Images/cancel.png"
#define CM_INFO        "Images/aide.png"
#define CM_NEWGAME        "Images/nouveauJeu.png"
#define CM_NEWNETGAME        "Images/newNetGame.png"
#define CM_OPENGAME    "Images/ouvrirJeu.png"
#define CM_SAVEGAME    "Images/saveGame.png"
#define CM_EXITGAME    "Images/quitter.png"
#define CM_RECYCLING        "Images/recycling.png"
#define CM_NEXTPLAYER    "Images/nextPlayer.png"
#define CM_RECYCLINGFINISHED    "Images/recyclingFinished.png"
#define CM_ATTACK1    "Images/attackOne.png"
#define CM_ATTACK2    "Images/attackTwo.png"
#define CM_ATTACK3    "Images/attackThree.png"
#define CM_DEFENSE1    "Images/defendOne.png"
#define CM_DEFENSE2    "Images/defendTwo.png"
#define CM_SHIFT    "Images/moveArmies.png"
#define CM_INVADE1    "Images/moveOne.png"
#define CM_INVADE5    "Images/moveFive.png"
#define CM_INVADE10    "Images/moveTen.png"
#define CM_RETREAT1    "Images/moveBackOne.png"
#define CM_RETREAT5    "Images/moveBackFive.png"
#define CM_RETREAT10    "Images/moveBackTen.png"
#define CM_INVASIONFINISHED    "Images/moveFinish.png"
#define CM_PREFERENCES    "Images/preferences.png"

#endif // KSIRK_GLOBAL_H

