/* simple stdint.h-style replacement without any conflicts */

#ifndef QINT_H
#define QINT_H

typedef unsigned char		my_uint8_t;
typedef unsigned short int	my_uint16_t;
typedef unsigned int		my_uint32_t;

typedef char                    my_int8_t;
typedef short int               my_int16_t;
typedef int                     my_int32_t;

#endif

