<?xml version="1.0" encoding="UTF-8"?>
<gui name="ksirkskineditor"
     version="1"
     xmlns="http://www.kde.org/standards/kxmlgui/1.0"
     xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
     xsi:schemaLocation="http://www.kde.org/standards/kxmlgui/1.0
                         http://www.kde.org/standards/kxmlgui/1.0/kxmlgui.xsd">

<MenuBar>
  <Menu name="game"><text>&amp;Game</text>
    <Action name="game_new" append="new_merge" />
    <Action name="game_load" append="new_merge" />
    <Action name="game_load_recent" append="new_merge" />
    <Action name="game_save" append="new_merge" />
    <Separator/>
    <Action name="game_quit" append="new_merge" />
  </Menu>
  <Menu name="settings"><text>&amp;Settings</text>
  </Menu>
  <Menu name="help"><text>&amp;Help</text>
  </Menu>
</MenuBar>

<ToolBar name="mainToolBar" noMerge="1"><text>Main Toolbar</text>
  <Action name="game_new" />
  <Action name="game_load" />
  <Action name="game_load_recent" />
  <Action name="game_save" />
  <Separator/>
  <Action name="view_zoom_in" />
  <Action name="view_zoom_out" />
  <Separator/>
</ToolBar>

</gui>
